from texas_hold_em_utils import Card
from texas_hold_em_utils.sklansky import sklansky_playable_position, sklansky_rank


def main():
    while True:
        print("Enter the value of the first card: ")
        value1 = str(input())
        if value1 not in ["2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K", "A"]:
            break
        print("Enter the suit of the first card: ")
        suit1 = str(input())
        if suit1 not in ["H", "D", "C", "S"]:
            break
        print("Enter the value of the second card: ")
        value2 = str(input())
        if value2 not in ["2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K", "A"]:
            break
        print("Enter the suit of the second card: ")
        suit2 = str(input())
        if suit2 not in ["H", "D", "C", "S"]:
            break
        card1 = Card().from_str(value1, suite_lookup(suit1))
        card2 = Card().from_str(value2, suite_lookup(suit2))
        print(f"Your cards are {card1.name()} and {card2.name()}")
        rank = sklansky_rank(card1, card2)
        print(f"Your rank is {rank}")
        print(f"Your position is {sklansky_playable_position(rank)}")


def suite_lookup(suit):
    if suit == "H":
        return "Hearts"
    if suit == "D":
        return "Diamonds"
    if suit == "C":
        return "Clubs"
    if suit == "S":
        return "Spades"
    return "Invalid suit"


if __name__ == "__main__":
    main()



def sklansky_playable_position(position: int):
    """
    Returns the playable positions for a given Sklansky rank
    https://en.wikipedia.org/wiki/Texas_hold_%27em_starting_hands#Sklansky_hand_groups
    :param position: int from 1 to 9
    :return: the positions and situations where the hand is playable
    """
    if position < 5:
        return "Early, Middle, Late"
    if position == 5:
        return "Early (loose/passive), Middle, Late"
    if position == 6:
        return "Middle (loose/passive), Late"
    if position == 7:
        return "Late if no bettors"
    if position == 8:
        return "Call if late and no bettors"
    return "Not playable"


def sklansky_rank(card1, card2):
    """
    Returns the Sklansky rank of the two hole cards passed in
    https://en.wikipedia.org/wiki/Texas_hold_%27em_starting_hands#Sklansky_hand_groups
    :param card1: a Card object
    :param card2: a (different) Card object
    :return: an int from 1 to 9, 1 being the best hand and 9 being the worst
    """
    higher_card = card1 if card1.rank > card2.rank else card2
    lower_card = card1 if card1.rank < card2.rank else card2
    if is_rank_1(higher_card, lower_card):
        return 1
    if is_rank_2(higher_card, lower_card):
        return 2
    if is_rank_3(higher_card, lower_card):
        return 3
    if is_rank_4(higher_card, lower_card):
        return 4
    if is_rank_5(higher_card, lower_card):
        return 5
    if is_rank_6(higher_card, lower_card):
        return 6
    if is_rank_7(higher_card, lower_card):
        return 7
    if is_rank_8(higher_card, lower_card):
        return 8
    return 9
    

def is_rank_1(higher_card, lower_card):
    if higher_card.rank == lower_card.rank and higher_card.rank >= 9:
        return True
    # ace king suited
    if (higher_card.suit == lower_card.suit) and (higher_card.rank == 12 and lower_card.rank == 11):
        return True
    return False
    

def is_rank_2(higher_card, lower_card):
    # ace king offsuit
    if higher_card.rank == 12 and lower_card.rank == 11:
        return True
    # ace queen/jack suited
    if (higher_card.suit == lower_card.suit) and (higher_card.rank == 12 and lower_card.rank >= 9):
        return True
    # king queen suited
    if (higher_card.suit == lower_card.suit) and (higher_card.rank == 11 and lower_card.rank == 10):
        return True
    # tens
    if higher_card.rank == 8 and lower_card.rank == 8:
        return True
    return False


def is_rank_3(higher_card, lower_card):
    # ace queen offsuit
    if higher_card.rank == 12 and lower_card.rank == 10:
        return True
    # jack king/queen suited
    if (higher_card.suit == lower_card.suit) and (higher_card.rank >= 10 and lower_card.rank == 9):
        return True
    # ace/jack ten suited
    if (higher_card.suit == lower_card.suit) and (higher_card.rank in [12, 9] and lower_card.rank == 8):
        return True
    # nines
    if higher_card.rank == 7 and lower_card.rank == 7:
        return True

    return False


def is_rank_4(higher_card, lower_card):
    # ace jack offsuit
    if higher_card.rank == 12 and lower_card.rank == 9:
        return True
    # king queen offsuit
    if higher_card.rank == 11 and lower_card.rank == 10:
        return True
    # ten king/queen suited
    if (higher_card.suit == lower_card.suit) and (higher_card.rank >= 10 and lower_card.rank == 8):
        return True
    # nine ten/jack suited
    if (higher_card.suit == lower_card.suit) and (higher_card.rank in [9, 8] and lower_card.rank >= 7):
        return True
    # nine eight suited
    if (higher_card.suit == lower_card.suit) and (higher_card.rank == 7 and lower_card.rank == 6):
        return True
    # eights
    if higher_card.rank == 6 and lower_card.rank == 6:
        return True
    
    return False


def is_rank_5(higher_card, lower_card):
    # jack king/queen offsuit
    if higher_card.rank >= 10 and lower_card.rank == 9:
        return True
    # jack ten offsuit
    if higher_card.rank == 9 and lower_card.rank == 8:
        return True
    # ace any suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 12:
        return True
    # queen nine suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 10 and lower_card.rank == 7:
        return True
    # ten eight suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 8 and lower_card.rank == 6:
        return True
    # seven eight/nine suited
    if higher_card.suit == lower_card.suit and higher_card.rank in [6, 7] and lower_card.rank == 5:
        return True
    # sevens
    if higher_card.rank == 5 and lower_card.rank == 5:
        return True
    # seven six suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 5 and lower_card.rank == 4:
        return True

    return False


def is_rank_6(higher_card, lower_card):
    # ace/king/queen ten offsuit
    if higher_card.rank >= 10 and lower_card.rank == 8:
        return True
    # king nine suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 11 and lower_card.rank == 7:
        return True
    # jack eight suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 9 and lower_card.rank == 6:
        return True
    # eight six suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 6 and lower_card.rank == 4:
        return True
    # sixes
    if higher_card.rank == 4 and lower_card.rank == 4:
        return True
    # seven five suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 5 and lower_card.rank == 3:
        return True
    # fives
    if higher_card.rank == 3 and lower_card.rank == 3:
        return True
    # five four suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 3 and lower_card.rank == 2:
        return True

    return False


def is_rank_7(higher_card, lower_card):
    # king any suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 11:
        return True
    # queen eight suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 10 and lower_card.rank == 6:
        return True
    # ten seven suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 8 and lower_card.rank == 5:
        return True
    # jack/ten nine offsuit
    if higher_card.rank in [9, 8] and lower_card.rank == 7:
        return True
    # nine eight offsuit
    if higher_card.rank == 7 and lower_card.rank == 6:
        return True
    # six five/four suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 4 and lower_card.rank in [3, 2]:
        return True
    # five/four three suited
    if higher_card.suit == lower_card.suit and higher_card.rank in [3, 2] and lower_card.rank == 1:
        return True
    # any pair
    if higher_card.rank == lower_card.rank:
        return True

    return False


def is_rank_8(higher_card, lower_card):
    # ace/king/queen nine offsuit
    if higher_card.rank >= 10 and lower_card.rank == 7:
        return True
    # jack/ten eight offsuit
    if higher_card.rank in [9, 8] and lower_card.rank == 6:
        return True
    # jack seven suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 9 and lower_card.rank == 5:
        return True
    # nine six suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 7 and lower_card.rank == 4:
        return True
    # eight five suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 6 and lower_card.rank == 3:
        return True
    # seven four suited
    if higher_card.suit == lower_card.suit and higher_card.rank == 5 and lower_card.rank == 2:
        return True
    # four/three two suited
    if higher_card.suit == lower_card.suit and higher_card.rank in [2, 1] and lower_card.rank == 0:
        return True
    # eight seven offsuit
    if higher_card.rank == 6 and lower_card.rank == 5:
        return True
    # seven six offsuit
    if higher_card.rank == 5 and lower_card.rank == 4:
        return True
    # six five offsuit
    if higher_card.rank == 4 and lower_card.rank == 3:
        return True
    # five four offsuit
    if higher_card.rank == 3 and lower_card.rank == 2:
        return True

    return False


    


from texas_hold_em_utils.preflop_stats_repository import PreflopStatsRepository


def test_get_win_rate():
    repo = PreflopStatsRepository()
    win_rate_info = repo.get_win_rate(0, 1, True, 2)

    assert win_rate_info['player_count'] == 2



from texas_hold_em_utils.card import Card
from texas_hold_em_utils.player import AllInPreFlopPlayer, LimpPlayer


def test_all_in_preflop_good_hand():
    player = AllInPreFlopPlayer(0, 1000, 0.5)
    # pocket aces should be above 50% win rate in a 2 player game
    player.hand_of_two.add_card(Card().from_ints(12, 0))
    player.hand_of_two.add_card(Card().from_ints(12, 1))

    action = player.decide(0, 0, 20, 20, [], 2)

    assert action[0] == "raise"
    assert action[1] == 1000


def test_all_in_preflop_bad_hand():
    player = AllInPreFlopPlayer(0, 1000, 0.5)
    # 2 7 off should be below 50% win rate in a 2 player game
    player.hand_of_two.add_card(Card().from_ints(0, 0))
    player.hand_of_two.add_card(Card().from_ints(5, 1))

    action = player.decide(0, 0, 20, 20, [], 2)

    assert action[0] == "fold"
    assert action[1] == 0


def test_all_in_preflop_bad_hand_big_blind():
    player = AllInPreFlopPlayer(0, 1000, 0.5)
    # 2 7 off should be below 50% win rate in a 2 player game
    player.hand_of_two.add_card(Card().from_ints(0, 0))
    player.hand_of_two.add_card(Card().from_ints(5, 1))
    player.round_bet = 20

    action = player.decide(0, 40, 20, 20, [], 2)

    assert action[0] == "check"
    assert action[1] == 0


def test_all_in_preflop_but_its_post_flop():
    player = AllInPreFlopPlayer(0, 1000, 0.5)
    # 2 7 off should be below 50% win rate in a 2 player game
    player.hand_of_two.add_card(Card().from_ints(0, 0))
    player.hand_of_two.add_card(Card().from_ints(5, 1))

    action = player.decide(1, 0, 20, 20, [], 2)

    assert action[0] == "check"
    assert action[1] == 0


def test_limper_no_bet():
    player = LimpPlayer(0, 1000, 0.5)
    player.round_bet = 20

    action = player.decide(0, 0, 20, 20, [], 2)

    assert action[0] == "check"
    assert action[1] == 0


def test_limper_already_all_in():
    player = LimpPlayer(0, 1000, 0.5)
    player.round_bet = 1000
    player.chips = 0

    action = player.decide(0, 0, 1000, 1000, [], 2)

    assert action[0] == "check"
    assert action[1] == 0


def test_limper_good_hand_preflop():
    player = LimpPlayer(0, 1000, 0.5)
    # pocket aces should be above 50% win rate in a 2 player game
    player.hand_of_two.add_card(Card().from_ints(12, 0))
    player.hand_of_two.add_card(Card().from_ints(12, 1))

    action = player.decide(0, 0, 20, 20, [], 2)

    assert action[0] == "call"
    assert action[1] == 20


def test_limper_good_hand_postflop():
    player = LimpPlayer(0, 1000, 0.5)
    player.hand_of_two.add_card(Card().from_ints(12, 0))
    player.hand_of_two.add_card(Card().from_ints(12, 1))
    player.round_bet = 20
    # pocket aces with some random unsuited community cards should win against most hands
    community = [Card().from_ints(0, 0), Card().from_ints(8, 1), Card().from_ints(4, 2)]

    action = player.decide(1, 60, 40, 20, community, 2)

    assert action[0] == "call"
    assert action[1] == 20


def test_limper_bad_hand_postflop():
    player = LimpPlayer(0, 1000, 0.5)
    player.hand_of_two.add_card(Card().from_ints(1, 0))
    player.hand_of_two.add_card(Card().from_ints(5, 1))
    player.round_bet = 20
    # pocket aces with some random unsuited community cards should win against most hands
    community = [Card().from_ints(0, 0), Card().from_ints(8, 1), Card().from_ints(4, 2)]

    action = player.decide(1, 60, 40, 20, community, 2)

    assert action[0] == "fold"
    assert action[1] == 0


def test_limper_bad_hand_preflop():
    player = LimpPlayer(0, 1000, 0.5)
    # 2 7 off should be below 50% win rate in a 2 player game
    player.hand_of_two.add_card(Card().from_ints(0, 0))
    player.hand_of_two.add_card(Card().from_ints(5, 1))

    action = player.decide(0, 0, 20, 20, [], 2)

    assert action[0] == "fold"
    assert action[1] == 0


def test_limper_good_hand_preflop_variable():
    player = LimpPlayer(0, 1000, [0.5, 1.0, 1.0, 1.0])
    # pocket aces should be above 50% win rate in a 2 player game
    player.hand_of_two.add_card(Card().from_ints(12, 0))
    player.hand_of_two.add_card(Card().from_ints(12, 1))

    action = player.decide(0, 0, 20, 20, [], 2)

    assert action[0] == "call"
    assert action[1] == 20


def test_limper_good_hand_postflop_variable():
    player = LimpPlayer(0, 1000, [1.0, 0.5, 1.0, 1.0])
    player.hand_of_two.add_card(Card().from_ints(12, 0))
    player.hand_of_two.add_card(Card().from_ints(12, 1))
    player.round_bet = 20
    # pocket aces with some random unsuited community cards should win against most hands
    community = [Card().from_ints(0, 0), Card().from_ints(8, 1), Card().from_ints(4, 2)]

    action = player.decide(1, 60, 40, 20, community, 2)

    assert action[0] == "call"
    assert action[1] == 20


def test_limper_bad_hand_postflop_variable():
    player = LimpPlayer(0, 1000, [0.0, 0.5, 0.0, 0.0])
    player.hand_of_two.add_card(Card().from_ints(1, 0))
    player.hand_of_two.add_card(Card().from_ints(5, 1))
    player.round_bet = 20
    # pocket aces with some random unsuited community cards should win against most hands
    community = [Card().from_ints(0, 0), Card().from_ints(8, 1), Card().from_ints(4, 2)]

    action = player.decide(1, 60, 40, 20, community, 2)

    assert action[0] == "fold"
    assert action[1] == 0


def test_limper_bad_hand_preflop_variable():
    player = LimpPlayer(0, 1000, [0.5, 0.0, 0.0, 0.0])
    # 2 7 off should be below 50% win rate in a 2 player game
    player.hand_of_two.add_card(Card().from_ints(0, 0))
    player.hand_of_two.add_card(Card().from_ints(5, 1))

    action = player.decide(0, 0, 20, 20, [], 2)

    assert action[0] == "fold"
    assert action[1] == 0



from texas_hold_em_utils.deck import Deck


def test_new_draw():
    deck = Deck()
    card = deck.draw()
    assert card.rank == 12
    assert card.suit == 3
    assert len(deck.cards) == 51


def test_all_cards_present():
    deck = Deck()
    for suit in [3, 2, 1, 0]:
        for rank in [12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0]:
            card = deck.draw()
            assert card.rank == rank
            assert card.suit == suit
    assert len(deck.cards) == 0


def test_shuffle():
    deck = Deck()
    deck.shuffle()
    assert len(deck.cards) == 52
    card = deck.draw()
    assert len(deck.cards) == 51
    for card2 in deck.cards:
        assert not (card.rank == card2.rank and card.suit == card2.suit)



from texas_hold_em_utils.card import Card
from texas_hold_em_utils.hands import HandOfFive


def test_compare_hands_same_rank():
    community_cards = [
        Card().from_str("A", "Hearts"),
        Card().from_str("A", "Spades"),
        Card().from_str("4", "Clubs"),
        Card().from_str("6", "Diamonds"),
        Card().from_str("9", "Hearts")
    ]
    # 2 pair
    hand1 = HandOfFive([
        Card().from_str("K", "Hearts"),
        Card().from_str("K", "Spades")
    ], community_cards)
    # 2 pair
    hand2 = HandOfFive([
        Card().from_str("Q", "Hearts"),
        Card().from_str("Q", "Spades")
    ], community_cards)

    assert hand1 > hand2


def test_compare_hands_diff_rank():
    community_cards = [
        Card().from_str("A", "Hearts"),
        Card().from_str("A", "Spades"),
        Card().from_str("4", "Clubs"),
        Card().from_str("6", "Diamonds"),
        Card().from_str("9", "Hearts")
    ]
    # 2 pair
    hand1 = HandOfFive([
        Card().from_str("K", "Hearts"),
        Card().from_str("K", "Spades")
    ], community_cards)
    # full house
    hand2 = HandOfFive([
        Card().from_str("A", "Clubs"),
        Card().from_str("9", "Clubs")
    ], community_cards)

    assert hand1 < hand2
